@@ -144,17 +144,39 @@ class buildcmd(cmd.Cmd):
     def do_adddir(self, directory, perm=0o777):
         '''Create a directory'''
         from os import makedirs
-        makedirs(join(self.builddir, directory), perm)  # lint:ok
+        makedirs(join(self.builddir, directory), int(perm, 8))
         print('Creted {} inside {}'.format(directory, self.builddir))
 
-    def do_install(self, rsrc, rdst, perm=0o755, owner='root'):
+    def do_install(self, rsrc, rdst, perm=0o755, uid=None, gid=None):
         '''Copy the src file to dst'''
-        from shutil import copyfile
+        from shutil import copyfile, chown
+        from os import chmod
         src = join(self.srcdir, rsrc)  # lint:ok
         dst = join(self.builddir, rdst)  # lint:ok
         copyfile(src, dst)
-        pass
+        chmod(dst, int(perm, 8))
+        chown(dst, user=uid, group=gid)
 
+    def do_shell(self, *command):
+        from subprocess import call
+        from upe import config
+        if config.building.shell:
+            if config.building.ask:
+                print('Tried to run ' + command)
+                ans = input('Allow (Y/N)? ')
+                if ans == 'Y' or ans == 'y':
+                    print('Allowed')
+                    call(command, shell=True)
+                else:
+                    print('Denied')
+            else:
+                print('Run command ' + command)
+                call(command, shell=True)
+        else:
+            print('Denied command execution: ' + command)
+
+def _parse(arg):
+    return tuple(map(str, arg.split()))
 
 def make(builds, manifest='manifest.json', builddir='build',
     srcdir='src'):
@@ -178,7 +200,7 @@ def make(builds, manifest='manifest.json', builddir='build',
     else:
         raise pkgmgmt.NotCompatible
     package = data['package']
-    validchars = tuple('abcdefghijklmnopqrstuvwxyz' + '0123456789' + '-_.')
+    validchars = ('abcdefghijklmnopqrstuvwxyz' + '0123456789' + '-_.',)
     pkgcode = list(package.lower())
     n = -1
     for l in pkgcode:
@@ -190,9 +212,9 @@ def make(builds, manifest='manifest.json', builddir='build',
         l = None
         while l != '':
             l = bs.readline
-            buildcmd(builddir, srcdir).onecmd(l)
+            buildcmd(builddir, srcdir).onecmd(*_parse(l))
     with open(join(builddir, 'manifest.json'), 'w') as destman:
         json.dump(data, destman)
     chdir(builddir)
     make_archive(join('..', '{}.pkg'.format(package)), 'bztar')
-    chdir('..')
\ No newline at end of file
+    chdir('..')
